/*******************************************************************************
 * Copyright (c) 2006 Oracle Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Cameron Bateman/Oracle - initial API and implementation
 *    
 ********************************************************************************/
package org.eclipse.jst.jsf.contentassist.internal.el;

import java.util.List;

import org.eclipse.jst.jsf.context.structureddocument.internal.provisional.IStructuredDocumentContext;

/**
 * Encapsulates an EL content assist completion prefix
 * 
 * @author cbateman
 *
 */
public abstract class ContentAssistStrategy
{
	/**
	 * ContentAssistStrategy type for a dot content assist completion
	 */
	public final static int  PREFIX_TYPE_DOT_COMPLETION = 0;
	/**
	 * ContentAssistStrategy type for an id content assist completion
	 */
	public final static int  PREFIX_TYPE_ID_COMPLETION = 1;
    /**
     * Completions for an empty expression
     */
    public final static int  PREFIX_TYPE_EMPTY_EXPRESSION = 2;
	
	private final int type;
	private final String value;
	
	/**
	 * @param type
	 * @param value
	 */
	public ContentAssistStrategy(final int type, final String value) 
	{
		super();
		this.type = type;
		this.value = value;
	}
    
	/**
	 * @return the type of this completion prefix as defined by
	 * one of the PREFIX_TYPE constants
	 */
	public final int getType() 
	{
		return type;
	}
    
	/**
	 * @return the token which this prefix is for
	 */
	public final String getValue() 
	{
		return value;
	}
    
    /**
     * @param context
     * @return the list of proposals for this strategy.  Contents of the list
     * must be of type ICompletionProposal
     */
    public abstract List getProposals(IStructuredDocumentContext context);
}
