/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.dialogfield;

import org.eclipse.jface.util.Assert;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.events.IHyperlinkListener;
import org.eclipse.ui.forms.widgets.FormToolkit;

/**
 * This is a wrapper to a dialog field, by adding a small button at the end. The
 * caller is responsible to provide the image or lable text and the button click
 * handler.
 * 
 * @author Yang Liu
 * @version
 * @see com.sybase.stf.jmt.pagedesigner.properties.celleditors.CellEditorWrapper
 */

public class DialogFieldWrapper implements DialogField, ISupportTextValue {
	private DialogField wrappedDialogField;

	private Button button;

	private boolean buttonEnabled;

	private Image buttonImage;

	private String buttonLabel;

	private IStringButtonAdapter adapter;

	/**
	 * 
	 */
	public DialogFieldWrapper(DialogField field, Image image) {
		super();
		if (!(field instanceof ISupportTextValue)) {
			throw new IllegalArgumentException(
					"Field must be ISupportTextValue");
		}
		wrappedDialogField = field;
		buttonImage = image;
	}

	/**
	 * 
	 */
	public DialogFieldWrapper(DialogField field, String label) {
		super();
		if (!(field instanceof ISupportTextValue)) {
			throw new IllegalArgumentException(
					"Field must be ISupportTextValue");
		}
		wrappedDialogField = field;
		buttonLabel = label;
	}

	public void setButtonAdapter(IStringButtonAdapter adapter) {
		this.adapter = adapter;
		this.updateButtonControl();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.ISupportTextValue#setTextWithoutUpdate(java.lang.String)
	 */
	public void setTextWithoutUpdate(String value) {
		((ISupportTextValue) wrappedDialogField).setTextWithoutUpdate(value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.ISupportTextValue#getText()
	 */
	public String getText() {
		return ((ISupportTextValue) wrappedDialogField).getText();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.ISupportTextValue#setText(java.lang.String)
	 */
	public void setText(String value) {
		((ISupportTextValue) wrappedDialogField).setText(value);
	}

	// --------------------------------------------------------------------------------------------
	// wrapped method to add the attached button
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.DialogField#doFillIntoGrid(org.eclipse.ui.forms.widgets.FormToolkit,
	 *      org.eclipse.swt.widgets.Composite, int)
	 */
	public Control[] doFillIntoGrid(FormToolkit toolkit, Composite parent,
			int nColumns) {
		Control[] wrappedControls = wrappedDialogField.doFillIntoGrid(toolkit,
				parent, nColumns - 1);
		Control[] result = new Control[wrappedControls.length];

		Control button = getButton(toolkit, parent);
		button.setLayoutData(gridDataForButton(1));

		System.arraycopy(wrappedControls, 0, result, 0, wrappedControls.length);
		result[result.length - 1] = this.button;
		return result;
	}

	/**
	 * @param span
	 * @return
	 */
	private GridData gridDataForButton(int span) {
		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gd.horizontalSpan = span;
		gd.widthHint = gd.heightHint = 18;
		return gd;
	}

	/**
	 * @param toolkit
	 * @param parent
	 * @return
	 */
	private Control getButton(FormToolkit toolkit, Composite parent) {
		if (button == null) {
			Assert.isNotNull(parent,
					"uncreated control requested with composite null"); //$NON-NLS-1$
			if (toolkit != null) {
				button = toolkit.createButton(parent, "", SWT.PUSH);
				if (buttonImage != null) {
					button.setImage(buttonImage);
				} else {
					button.setText(buttonLabel);
				}
			} else {
				button = new Button(parent, SWT.PUSH);
				if (buttonImage != null) {
					button.setImage(buttonImage);
				} else {
					button.setText(buttonLabel);
				}
			}
			button.setEnabled(isEnabled() && buttonEnabled);
			button.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
					buttonPressed();
				}

				public void widgetSelected(SelectionEvent e) {
					buttonPressed();
				}
			});

		}
		return button;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.DialogField#getLabelControl(org.eclipse.ui.forms.widgets.FormToolkit,
	 *      org.eclipse.swt.widgets.Composite)
	 */
	public Control getLabelControl(FormToolkit _formToolkit, Composite parent) {
		return wrappedDialogField.getLabelControl(_formToolkit, parent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#setHyperLink(org.eclipse.ui.forms.events.IHyperlinkListener)
	 */
	public void setHyperLink(IHyperlinkListener listener) {
		wrappedDialogField.setHyperLink(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#setLabelText(java.lang.String)
	 */
	public void setLabelText(String labeltext) {
		wrappedDialogField.setLabelText(labeltext);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#setDialogFieldChangeListener(org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogFieldChangeListener)
	 */
	public void setDialogFieldChangeListener(IDialogFieldChangeListener listener) {
		wrappedDialogField.setDialogFieldChangeListener(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#setDialogFieldApplyListener(org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogFieldApplyListener)
	 */
	public void setDialogFieldApplyListener(IDialogFieldApplyListener listener) {
		wrappedDialogField.setDialogFieldApplyListener(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#setFocus()
	 */
	public boolean setFocus() {
		return wrappedDialogField.setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#getNumberOfControls()
	 */
	public int getNumberOfControls() {
		return wrappedDialogField.getNumberOfControls() + 1;
	}

	public void setButtonEnabled(boolean enabled) {
		this.buttonEnabled = enabled;
		updateButtonControl();
	}

	public boolean isButtonEnabled() {
		return buttonEnabled;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {
		wrappedDialogField.setEnabled(enabled);
		updateButtonControl();
	}

	/**
	 * 
	 */
	private void updateButtonControl() {
		if (this.button != null) {
			this.button.setEnabled(this.isEnabled() && buttonEnabled
					&& adapter != null);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#isEnabled()
	 */
	public boolean isEnabled() {
		return wrappedDialogField.isEnabled();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#getAttachedData(java.lang.Object)
	 */
	public Object getAttachedData(Object key) {
		return wrappedDialogField.getAttachedData(key);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.IDialogField#putAttachedData(java.lang.Object,
	 *      java.lang.Object)
	 */
	public void putAttachedData(Object key, Object value) {
		wrappedDialogField.putAttachedData(key, value);
	}

	protected void buttonPressed() {
		if (adapter != null) {
			adapter.changeControlPressed(this);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.DialogField#handleGrabHorizontal()
	 */
	public void handleGrabHorizontal() {
		wrappedDialogField.handleGrabHorizontal();
	}

	public DialogField getWrappedDialogField() {
		return wrappedDialogField;
	}

	public boolean isRequired() {
		return wrappedDialogField.isRequired();
	}
}