/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.dialogfield;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jst.jsf.facesconfig.common.CommonPlugin;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;


/**
 * @author zpzhang
 */
public class JavaClassWizard extends Wizard {
	private static String STORE_SECTION = "JavaClassWizard"; //$NON-NLS-1$

	private JavaClassWizardPage mainPage;

	private String className, classArgs;

	private IProject project;

	private String superClass;

	private List interfaceList;

	private boolean autoOpenResource = true;

	public void setAutoOpenResource(boolean autoOpenResource) {
		this.autoOpenResource = autoOpenResource;
	}

	public JavaClassWizard(IProject project, String className) {
		this(project, className, null, null);
	}

	public JavaClassWizard(IProject project, String className,
			String superClass, List superInterfaces) {
		this.project = project;
		this.className = className;
		this.superClass = superClass;
		this.interfaceList = superInterfaces;
		IDialogSettings masterSettings = CommonPlugin.getDefault()
				.getDialogSettings();
		setDialogSettings(getSettingsSection(masterSettings));
		setWindowTitle(CommonPlugin
				.getResourceString("DialogField.JavaClassWizard.NewClass"));//$NON-NLS-1$
		setNeedsProgressMonitor(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages() {
		mainPage = new JavaClassWizardPage(project, className, superClass,
				interfaceList);
		addPage(mainPage);
		mainPage.init();
	}

	private IDialogSettings getSettingsSection(IDialogSettings master) {
		IDialogSettings setting = master.getSection(STORE_SECTION);
		if (setting == null) {
			setting = master.addNewSection(STORE_SECTION);
		}
		return setting;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish() {
		if (mainPage.getPackageText() != null
				&& mainPage.getPackageText().length() > 0) {
			StringBuffer buffer = new StringBuffer(mainPage.getPackageText());
			buffer.append(".");//$NON-NLS-1$
			buffer.append(mainPage.getTypeName());
			className = buffer.toString();
		} else {
			className = mainPage.getTypeName();
		}
		classArgs = mainPage.getClassArgs();
		IRunnableWithProgress op = new WorkspaceModifyOperation() {
			protected void execute(IProgressMonitor monitor)
					throws CoreException, InvocationTargetException,
					InterruptedException {
				mainPage.createType(monitor);
				IResource resource = mainPage.getModifiedResource();
				if (resource != null && autoOpenResource) {
					selectAndReveal(resource);
					if (project.hasNature(JavaCore.NATURE_ID)) {
						IJavaProject jProject = JavaCore.create(project);
						IJavaElement jElement = jProject.findElement(resource
								.getProjectRelativePath()
								.removeFirstSegments(1));
						if (jElement != null) {
							JavaUI.openInEditor(jElement);
						}
					} else if (resource instanceof IFile) {
						IWorkbenchPage page = PlatformUI.getWorkbench()
								.getActiveWorkbenchWindow().getActivePage();
						IDE.openEditor(page, (IFile) resource, true);
					}
				}
			}

		};
		try {
			getContainer().run(false, true, op);
		} catch (InvocationTargetException e) {
			e.printStackTrace(); // PDEPlugin.logException(e);
		} catch (InterruptedException e) {
			e.printStackTrace();// PDEPlugin.logException(e);
		}
		return true;
	}

	protected void selectAndReveal(IResource newResource) {
		BasicNewResourceWizard.selectAndReveal(newResource,
				getWorkbenchWindow());
	}

	private IWorkbenchWindow getWorkbenchWindow() {
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow();
	}

	protected ISchedulingRule getSchedulingRule() {
		return mainPage.getModifiedResource();
	}

	public String getClassName() {
		return className;
	}

	public String getClassNameWithArgs() {
		if (classArgs != null && classArgs.length() > 0) {
			StringBuffer buffer = new StringBuffer(className);
			buffer.append(":");//$NON-NLS-1$
			buffer.append(classArgs);
			return buffer.toString();
		}
		return getClassName();
	}
}
