/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.dialogfield;

import java.util.ArrayList;
import java.util.HashSet;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchScope;

/**
 * @author zpzhang
 */
public class JavaSearchScope implements IJavaSearchScope {
	private IProject project;

	private String superType;

	private HashSet allowedTypeSet;

	private IPath[] enclosingProjectsAndJars;

	private IProject[] relativeProjects;

	public JavaSearchScope(IProject project, String superType) {
		this.project = project;
		this.superType = superType;
		computeRelativeProjects();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.core.search.IJavaSearchScope#enclosingProjectsAndJars()
	 */
	public IPath[] enclosingProjectsAndJars() {
		if (enclosingProjectsAndJars == null) {
			ArrayList list = new ArrayList();
			for (int i = 0; i < relativeProjects.length; i++) {
				try {
					if (relativeProjects[i].hasNature(JavaCore.NATURE_ID)) {
						IJavaProject javaProject = JavaCore
								.create(relativeProjects[i]);
						IClasspathEntry[] classpath = javaProject
								.getResolvedClasspath(true);
						for (int j = 0; j < classpath.length; j++) {
							list.add(classpath[j].getPath());
						}
						list.add(javaProject.getPath());
					}
				} catch (CoreException e)// NOPMD
				{
					// skip the project.
				}
			}
			enclosingProjectsAndJars = (IPath[]) list.toArray(new IPath[(list
					.size())]);
		}
		return enclosingProjectsAndJars;
	}

	private void computeRelativeProjects() {
		try {
			IProject[] referencedProjects = project.getReferencedProjects();
			relativeProjects = new IProject[referencedProjects.length + 1];
			System.arraycopy(referencedProjects, 0, relativeProjects, 1,
					referencedProjects.length);
			relativeProjects[0] = project;
		} catch (CoreException e) {
			relativeProjects = new IProject[] { project };
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.core.search.IJavaSearchScope#encloses(java.lang.String)
	 */
	public boolean encloses(String resourcePath) {
		if (allowedTypeSet == null) {
			try {
				allowedTypeSet = findAllowedTypes(superType);
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
		if (allowedTypeSet == null) {
			allowedTypeSet = new HashSet();
		}
		int separatorIndex = resourcePath.indexOf(JAR_FILE_ENTRY_SEPARATOR);
		if (separatorIndex != -1) {
			String className = resourcePath.substring(separatorIndex + 1,
					resourcePath.length() - 6).replace('/', '.');
			if (allowedTypeSet.contains(className)) {
				return true;
			}
		} else if (allowedTypeSet.contains(resourcePath)) {
			return true;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.core.search.IJavaSearchScope#encloses(org.eclipse.jdt.core.IJavaElement)
	 */
	public boolean encloses(IJavaElement element) {
		return encloses(element.getPath().toString());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.core.search.IJavaSearchScope#includesBinaries()
	 */
	public boolean includesBinaries() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.core.search.IJavaSearchScope#includesClasspaths()
	 */
	public boolean includesClasspaths() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.core.search.IJavaSearchScope#setIncludesBinaries(boolean)
	 */
	public void setIncludesBinaries(boolean includesBinaries) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.core.search.IJavaSearchScope#setIncludesClasspaths(boolean)
	 */
	public void setIncludesClasspaths(boolean includesClasspaths) {
	}

	private HashSet findAllowedTypes(String _superType) throws CoreException {
		HashSet set = new HashSet();

		IProject[] projects = relativeProjects;

		for (int i = 0; i < projects.length; i++) {
			IType type = null;
			if (projects[i].hasNature(JavaCore.NATURE_ID)) {
				IJavaProject javaProject = JavaCore.create(projects[i]);
				if (_superType != null) {
					try {
						type = javaProject.findType(_superType);
						if (type != null) {
							ITypeHierarchy typeHierarchy = type
									.newTypeHierarchy(javaProject, null);
							IType[] subtypes = typeHierarchy
									.getAllSubtypes(type);
							for (int j = 0; j < subtypes.length; j++) {
								if (!subtypes[j].isBinary()) {
									set.add(subtypes[j].getPath().toString());
								} else {
									set
											.add(subtypes[j]
													.getFullyQualifiedName());
								}
							}
						}
					} catch (JavaModelException e) {
						e.printStackTrace();
					}
				}
			}
		}
		return set;
	}

	/**
	 * @return Returns the superType.
	 */
	public String getSuperType() {
		return superType;
	}

	/**
	 * @param superType
	 *            The superType to set.
	 */
	public void setSuperType(String superType) {
		this.superType = superType;
	}
}
