/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.dialogfield;

import org.eclipse.jst.jsf.facesconfig.common.CommonPlugin;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;


/**
 * Dialog field containing a label, text control and a button control.
 */
public class StringButtonDialogField extends StringDialogField {
	private Button browseButton;

	private String browseButtonLabel;

	private IStringButtonAdapter stringButtonAdapter;

	private boolean buttonEnabled;

	public StringButtonDialogField(IStringButtonAdapter adapter) {
		stringButtonAdapter = adapter;
		browseButtonLabel = CommonPlugin
				.getResourceString("DialogField.Browse");
		buttonEnabled = true;
	}

	/**
	 * Sets the label of the button.
	 */
	public void setButtonLabel(String label) {
		browseButtonLabel = label;
	}

	// ------ adapter communication

	/**
	 * Programmatical pressing of the button
	 */
	public void changeControlPressed() {
		stringButtonAdapter.changeControlPressed(this);
	}

	// ------- layout helpers

	/*
	 * @see DialogField#doFillIntoGrid
	 */
	public Control[] doFillIntoGrid(FormToolkit toolkit, Composite parent,
			int nColumns) {
		assertEnoughColumns(nColumns);

		Control requiredLabel = getRequiredLabelControl(toolkit, parent);
		requiredLabel.setLayoutData(gridDataForLabel(1));

		Control label = getLabelControl(toolkit, parent);
		label.setLayoutData(gridDataForLabel(1));

		Text text = getTextControl(toolkit, parent);
		text.setLayoutData(gridDataForText(nColumns - 3));
		Button button = getChangeControl(toolkit, parent);
		button.setLayoutData(gridDataForButton(toolkit, button, text, 1));

		return new Control[] { requiredLabel, label, text, button };
	}

	/*
	 * @see DialogField#getNumberOfControls
	 */
	public int getNumberOfControls() {
		return 4;
	}

	protected static GridData gridDataForButton(FormToolkit toolkit,
			Button button, Text text, int span) {
		GridData gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = false;
		gd.horizontalSpan = span;
		// gd.heightHint = SWTUtil.getButtonHeightHint(button);
		gd.widthHint = LayoutUtil.getButtonWidthHint(button);
		gd.heightHint = LayoutUtil.getButtonHeightHint(toolkit, text);
		return gd;
	}

	// ------- ui creation

	/**
	 * Creates or returns the created buttom widget.
	 * 
	 * @param parent
	 *            The parent composite or <code>null</code> if the widget has
	 *            already been created.
	 */
	public Button getChangeControl(FormToolkit toolkit, Composite parent) {
		if (browseButton == null) {
			assertCompositeNotNull(parent);
			if (toolkit != null) {
				browseButton = toolkit.createButton(parent, browseButtonLabel,
						SWT.PUSH);
			} else {
				browseButton = new Button(parent, SWT.PUSH);
				browseButton.setText(browseButtonLabel);
			}
			browseButton.setEnabled(isEnabled() && buttonEnabled);
			browseButton.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
					changeControlPressed();
				}

				public void widgetSelected(SelectionEvent e) {
					changeControlPressed();
				}
			});

		}
		return browseButton;
	}

	// ------ enable / disable management

	/**
	 * Sets the enable state of the button.
	 */
	public void enableButton(boolean enable) {
		if (isOkToUse(browseButton)) {
			browseButton.setEnabled(isEnabled() && enable);
		}
		buttonEnabled = enable;
	}

	/*
	 * @see DialogField#updateEnableState
	 */
	protected void updateEnableState() {
		super.updateEnableState();
		if (isOkToUse(browseButton)) {
			browseButton.setEnabled(isEnabled() && buttonEnabled);
		}
	}

	/**
	 * @return Returns the _stringButtonAdapter.
	 */
	public IStringButtonAdapter getStringButtonAdapter() {
		return stringButtonAdapter;
	}

	/**
	 * @param buttonAdapter
	 *            The _stringButtonAdapter to set.
	 */
	public void setStringButtonAdapter(IStringButtonAdapter buttonAdapter) {
		stringButtonAdapter = buttonAdapter;
	}
}
