/*******************************************************************************
 * Copyright (c) 2006 Oracle Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Cameron Bateman/Oracle - initial API and implementation
 *    
 ********************************************************************************/

package org.eclipse.jst.jsf.validation.internal.el;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class Activator extends AbstractUIPlugin {

//	private Map  _projectFaces = new HashMap();
	
	//The shared instance.
	private static Activator plugin;
	
	private static ResourceBundle	_resourceBundle;
	/**
	 * The constructor.
	 */
	public Activator() {
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 * @param context 
	 * @throws Exception 
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 * @param context 
	 * @throws Exception 
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
	}

	/**
	 * @return the resource bundle
	 */
	public ResourceBundle getResourceBundle()
	{
		return _resourceBundle;
	}
	
	/**
	 * Returns the shared instance.
	 *
	 * @return the shared instance.
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path.
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public ImageDescriptor getImageDescriptor(String path) {
		if (path == null) {
			return null;
		}

		ImageRegistry images = getImageRegistry();
		ImageDescriptor id = images.getDescriptor(path);
		if (id == null) {
			try {
				URL pluginBase = getBundle().getEntry("/");
				id = ImageDescriptor.createFromURL(new URL(pluginBase, "icons/"
						+ path));
				images.put(path, id);
			} catch (MalformedURLException ee) {
				// log.EditorPlugin.image.error=Image {0} not found.
				//log.error("log.msg", "log.EditorPlugin.image.error", path, ee);
			}
		}
		return id;
	}
	
	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not
	 * found.
	 * @param key the key used to look up the string
	 * @return the resource string if one is found matching key or key is returned if it is not
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * @param message
	 * @param t
	 */
	public static void log(String message, Throwable t)
    {
	    ILog log = plugin.getLog();
        log.log(new Status(IStatus.ERROR, 
                            plugin.getBundle().getSymbolicName(),
                            0,
                            message,
                            t));
    }

}
